@@ -160,23 +160,27 @@ impl InstantiateToolEnvironmentSpec {
 
         // Determine the cache key for the environment.
         let cache_key = self.cache_key();
+        let name = self.requirement.0.as_source().to_string();
 
         // Construct the prefix for the tool environment.
         let prefix = Prefix::create(command_queue.cache_dirs().build_backends().join(cache_key))
-            .map_err(InstantiateToolEnvironmentError::CreatePrefix)?;
+            .map_err(InstantiateToolEnvironmentError::CreatePrefix)
+            .map_err(CommandDispatcherError::Failed)?;
 
         // Acquire a lock on the tool prefix.
         let mut prefix_guard = AsyncPrefixGuard::new(prefix.path())
             .and_then(|guard| guard.write())
             .await
-            .map_err(InstantiateToolEnvironmentError::AcquireLock)?;
+            .map_err(InstantiateToolEnvironmentError::AcquireLock)
+            .map_err(CommandDispatcherError::Failed)?;
 
         // If the environment already exists, we can return early.
         if prefix_guard.is_ready() {
             prefix_guard
                 .finish()
                 .await
-                .map_err(InstantiateToolEnvironmentError::ReleaseLock)?;
+                .map_err(InstantiateToolEnvironmentError::ReleaseLock)
+                .map_err(CommandDispatcherError::Failed)?;
             return Ok(prefix);
         }
 
@@ -184,7 +188,8 @@ impl InstantiateToolEnvironmentSpec {
         prefix_guard
             .begin()
             .await
-            .map_err(InstantiateToolEnvironmentError::UpdateLock)?;
+            .map_err(InstantiateToolEnvironmentError::UpdateLock)
+            .map_err(CommandDispatcherError::Failed)?;
 
         let build_api_version_nameless_spec = NamelessMatchSpec {
             version: Some(PIXI_BUILD_API_VERSION_SPEC.clone()),
@@ -205,7 +210,7 @@ impl InstantiateToolEnvironmentSpec {
         let target_platform = self.build_environment.host_platform;
         let solved_environment = command_queue
             .solve_pixi_environment(PixiEnvironmentSpec {
-                name: Some(self.requirement.0.as_normalized().to_string()),
+                name: Some(name.clone()),
                 dependencies: self
                     .additional_requirements
                     .into_specs()
@@ -214,17 +219,30 @@ impl InstantiateToolEnvironmentSpec {
                 constraints,
                 build_environment: self.build_environment,
                 exclude_newer: self.exclude_newer,
-                channel_config: self.channel_config,
-                channels: self.channels,
-                enabled_protocols: self.enabled_protocols,
+                channel_config: self.channel_config.clone(),
+                channels: self.channels.clone(),
+                enabled_protocols: self.enabled_protocols.clone(),
                 installed: Vec::new(), // Install from scratch
                 channel_priority: ChannelPriority::default(),
-                variants: self.variants,
+                variants: self.variants.clone(),
                 strategy: SolveStrategy::default(),
             })
             .await
             .map_err_with(Box::new)
             .map_err_with(InstantiateToolEnvironmentError::SolveEnvironment)?;
+
+        if tracing::event_enabled!(tracing::Level::INFO) {
+            let packages_str = solved_environment
+                .iter()
+                .map(|p| {
+                    let record = p.package_record();
+                    format!("{}={}", record.name.as_normalized(), record.version)
+                })
+                .collect::<Vec<_>>()
+                .join(", ");
+            tracing::info!("Packages in tool environment: {}", packages_str);
+        }
+
         // Ensure that solution contains matching api version package
         if !solved_environment
             .iter()
@@ -240,20 +258,27 @@ impl InstantiateToolEnvironmentSpec {
         // Install the environment
         command_queue
             .install_pixi_environment(InstallPixiEnvironmentSpec {
+                name,
                 records: solved_environment,
                 prefix: prefix.clone(),
                 installed: None,
-                platform: target_platform,
+                target_platform,
                 force_reinstall: Default::default(),
+                channels: self.channels,
+                channel_config: self.channel_config,
+                variants: self.variants,
+                enabled_protocols: self.enabled_protocols,
             })
             .await
+            .map_err_with(Box::new)
             .map_err_with(InstantiateToolEnvironmentError::InstallEnvironment)?;
 
         // Mark the environment as finished.
         prefix_guard
             .finish()
             .await
-            .map_err(InstantiateToolEnvironmentError::UpdateLock)?;
+            .map_err(InstantiateToolEnvironmentError::UpdateLock)
+            .map_err(CommandDispatcherError::Failed)?;
 
         Ok(prefix)
     }
@@ -280,7 +305,7 @@ pub enum InstantiateToolEnvironmentError {
 
     #[error(transparent)]
     #[diagnostic(transparent)]
-    InstallEnvironment(InstallPixiEnvironmentError),
+    InstallEnvironment(Box<InstallPixiEnvironmentError>),
 
     #[error("The environment for the build backend package (`{} {}`) does not depend on `{}`. Without this package pixi has no way of knowing the API to use to communicate with the backend.", .build_backend.0.as_normalized(), .build_backend.1.to_string(), PIXI_BUILD_API_VERSION_NAME.as_normalized())]
     #[diagnostic(help(
