@@ -213,7 +213,7 @@ pub async fn resolve_pypi(
     pb: &ProgressBar,
     project_root: &Path,
     prefix_updater: CondaPrefixUpdater,
-    platform_repodata_records: Arc<PixiRecordsByName>,
+    repodata_building_records: miette::Result<Arc<PixiRecordsByName>>,
     project_env_vars: HashMap<EnvironmentName, EnvironmentVars>,
     environment_name: Environment<'_>,
     disallow_install_conda_prefix: bool,
@@ -321,8 +321,8 @@ pub async fn resolve_pypi(
     let requires_python = uv_resolver::RequiresPython::from_specifiers(
         &uv_pep440::VersionSpecifiers::from(python_specifier),
     );
-    tracing::info!(
-        "using requires python specifier (this may differ from the above): {}",
+    tracing::debug!(
+        "using requires-python specifier (this may differ from the above): {}",
         requires_python
     );
 
@@ -414,7 +414,7 @@ pub async fn resolve_pypi(
         prefix_updater,
         project_env_vars,
         environment_name,
-        platform_repodata_records.records.clone(),
+        repodata_building_records.map(|r| r.records.clone()),
         pypi_options.no_build_isolation.clone(),
         &lazy_build_dispatch_dependencies,
         disallow_install_conda_prefix,
