@@ -403,7 +403,7 @@ impl WorkspaceMut {
             self.save_inner().await.into_diagnostic()?;
         }
 
-        let mut updated_lock_file = LockFileDerivedData {
+        let updated_lock_file = LockFileDerivedData {
             workspace: self.workspace(),
             lock_file,
             package_cache,
@@ -428,7 +428,7 @@ impl WorkspaceMut {
                 .prefix(
                     &self.workspace().default_environment(),
                     UpdateMode::Revalidate,
-                    ReinstallPackages::default(),
+                    &ReinstallPackages::default(),
                 )
                 .await?;
         }
